@@ -176,7 +176,7 @@ menustruct Stipples[] = {
 };
 
 menustruct TextMenu[] = {
-	{"Text Size", action(gettsize, &areastruct.textscale)},
+	{"Text Size", action(gettsize, &areawin->textscale)},
 	{"Font", submenu(Fonts)},
 	{"Style", submenu(FontStyles)},
 	{"Encoding", submenu(FontEncodings)},
@@ -187,7 +187,7 @@ menustruct TextMenu[] = {
 	{" ", noaction},
 	{"Set LaTeX Mode", action(setjustbit, Number(LATEXLABEL))},
 	{" ", noaction},
-	{"Make Label (t)", action(starttext, Number(NORMAL))},
+	{"Make Label (t)", action(changetool, Number(XCF_Text))},
 };
 
 menustruct PolyMenu[] = {
@@ -197,8 +197,8 @@ menustruct PolyMenu[] = {
 	{" ", noaction},
 	{"Object size", action(getosize, NULL)},
 	{"Parameters", submenu(Parameterize)},
-	{"Center Object", setaction(toggle, &areastruct.center)},
-	{"Manhattan Draw", action(toggle, &areastruct.manhatn)},
+	{"Center Object", setaction(toggle, &areawin->center)},
+	{"Manhattan Draw", action(toggle, &areawin->manhatn)},
 	{"Polygon Edit", submenu(BoxEditStyles)},
 };
 
@@ -212,38 +212,38 @@ menustruct FilterMenu[] = {
 };
 
 menustruct RotateMenu[] = {
-	{"Flip Horizontal (f)", action(startrotate, Number(512))},
-	{"Flip Vertical (F)", action(startrotate, Number(1024))},
+	{"Flip Horizontal (f)", action(exec_or_changetool, Number(XCF_Flip_X))},
+	{"Flip Vertical (F)", action(exec_or_changetool, Number(XCF_Flip_Y))},
 	{" ", noaction},
-	{"Rotate CW 90", action(startrotate, Number(90))},
-	{"Rotate CW 45", action(startrotate, Number(45))},
-	{"Rotate CW 30", action(startrotate, Number(30))},
-	{"Rotate CW 15 (r)", action(startrotate, Number(15))},
-	{"Rotate CW 5 (o)", action(startrotate, Number(3))},
-	{"Rotate CW 1", action(startrotate, Number(1))},
+	{"Rotate CW 90", action(rotatetool, Number(90))},
+	{"Rotate CW 45", action(rotatetool, Number(45))},
+	{"Rotate CW 30", action(rotatetool, Number(30))},
+	{"Rotate CW 15 (r)", action(rotatetool, Number(15))},
+	{"Rotate CW 5 (o)", action(rotatetool, Number(3))},
+	{"Rotate CW 1", action(rotatetool, Number(1))},
 	{" ", noaction},
-	{"Rotate CCW 90", action(startrotate, Number(-90))},
-	{"Rotate CCW 45", action(startrotate, Number(-45))},
-	{"Rotate CCW 30", action(startrotate, Number(-30))}, 
-	{"Rotate CCW 15 (R)", action(startrotate, Number(-15))},
-	{"Rotate CCW 5 (O)", action(startrotate, Number(-5))},
-	{"Rotate CCW 1", action(startrotate, Number(-1))},
+	{"Rotate CCW 90", action(rotatetool, Number(-90))},
+	{"Rotate CCW 45", action(rotatetool, Number(-45))},
+	{"Rotate CCW 30", action(rotatetool, Number(-30))}, 
+	{"Rotate CCW 15 (R)", action(rotatetool, Number(-15))},
+	{"Rotate CCW 5 (O)", action(rotatetool, Number(-5))},
+	{"Rotate CCW 1", action(rotatetool, Number(-1))},
 };
 
 menustruct LineMenu[] = {
 	{"Global Linewidth", action(getwirewidth, NULL)},
 	{"Wire Linewidth", action(getwwidth, NULL)},
-}
+};
 
 menustruct GridMenu[] = {
-	{"Grid", setaction(toggle, &areastruct.gridon)},
-	{"Axes", setaction(toggle, &areastruct.axeson)},
+	{"Grid", setaction(toggle, &areawin->gridon)},
+	{"Axes", setaction(toggle, &areawin->axeson)},
 	{"Grid spacing", action(getgridspace, NULL)},
 	{"Grid type/display", submenu(GridStyles)},
 };
 
 menustruct SnapMenu[] = {
-	{"Snap-to", setaction(toggle, &areastruct.snapto)},
+	{"Snap-to", setaction(toggle, &areawin->snapto)},
         {"Snap spacing", action(getsnapspace, NULL)},
 };
 
@@ -255,9 +255,9 @@ menustruct PinConvert[] = {
 };
 
 menustruct SchemaMenu[] = {
-	{"Make Pin (T)", action(starttext, Number(LOCAL))},
-	{"Make Info Pin (I)", action(starttext, Number(INFO))},
-	{"Make Global Pin (G)", action(starttext, Number(GLOBAL))},
+	{"Make Pin (T)", action(changetool, Number(XCF_Pin_Label))},
+	{"Make Info Pin (I)", action(changetool, Number(XCF_Info_Label))},
+	{"Make Global Pin (G)", action(changetool, Number(XCF_Pin_Global))},
 	{"Convert Label to...", submenu(PinConvert)},
 	{"Pin Visibility", action(setpinjustbit, Number(PINVISIBLE))},
 /*	{"Make Object Non-Schematic", action(maketrivial, NULL)}, */
@@ -275,9 +275,8 @@ menustruct SchemaMenu[] = {
 menustruct WindowMenu[] = {
 	{"Zoom In (Z)", action(zoominrefresh, NULL)},
 	{"Zoom Out (z)", action(zoomoutrefresh, NULL)},
-	{"Zoom Box", action(zoombox, NULL)},
 	{"Full View (v)", action(zoomview, NULL)},
-	{"Center Pan (p)",action(centerpan, NULL)},
+	{"Pan (p)", action(pantool, Number(6))},
 	{"Refresh ( )", action(refresh, NULL)},
 	{" ", noaction},
 	{"Library Directory (L)", action(startcatalog, Number(LIBLIB))},
@@ -307,10 +306,10 @@ menustruct FileMenu[] = {
 };
 
 menustruct OptionMenu[] = {
-	{"Alt Colors", action(inversecolor, &areastruct.invert)},
-	{"Show Bounding box", action(toggle, &areastruct.bboxon)},
-	{"Edit in place", setaction(toggle, &areastruct.editinplace)},
-	{"Show Pin Positions", action(toggle, &areastruct.pinpointon)},
+	{"Alt Colors", action(inversecolor, &areawin->invert)},
+	{"Show Bounding box", action(toggle, &areawin->bboxon)},
+	{"Edit in place", setaction(toggle, &areawin->editinplace)},
+	{"Show Pin Positions", action(toggle, &areawin->pinpointon)},
 	{"Enable XSchema", action(doxschema, Number(1))},
 #ifdef HAVE_XPM
 	{"Disable Toolbar", action(dotoolbar, NULL)},
@@ -327,20 +326,19 @@ menustruct EditMenu[] = {
 	{"Undo (u)", action(undo_call, NULL)},
 	{"Redo (U)", action(redo_call, NULL)},
 	{" ", noaction},
-	{"Delete (d)", action(startdelete, NULL)},
-	{"Copy (c)", action(startcopy, NULL)},
-	{"Edit (e)", action(startedit, NULL)},
+	{"Delete (d)", action(exec_or_changetool, Number(XCF_Delete))},
+	{"Copy (c)", action(exec_or_changetool, Number(XCF_Copy))},
+	{"Edit (e)", action(exec_or_changetool, Number(XCF_Edit))},
 	{"Rotate/Flip", submenu(RotateMenu)},
 	{"Deselect (x)", action(startdesel, NULL)},
 	{"Select filter", submenu(FilterMenu)},
-/*	{"Array", noaction}, */
-	{"Push selected (>)", action(startpush, NULL)},
+	{"Push selected (>)", action(exec_or_changetool, Number(XCF_Push))},
 	{"Pop hierarchy (<)", action(popobject, Number(0))},
 	{" ", noaction},
 	{"Make User Object (m)", action(selectsave, NULL)},
-	{"Make Arc (a)", action(startarc, NULL)},
-	{"Make Box (b)", action(startbox, NULL)},
-	{"Make Spline (s)", action(startspline, NULL)},
+	{"Make Arc (a)", action(changetool, Number(XCF_Arc))},
+	{"Make Box (b)", action(changetool, Number(XCF_Box))},
+	{"Make Spline (s)", action(changetool, Number(XCF_Spline))},
 	{"Join (j)", action(join, NULL)},
 };
 
@@ -358,29 +356,35 @@ short maxbuttons = sizeof(TopButtons) / sizeof(menustruct);
 #ifdef HAVE_XPM
 /* Toolbar buttons */
 toolbarstruct ToolBar[] = {
-	{"Box",    b_xpm,  toolaction(startbox, NULL),
+	{"Pan",	   pn_xpm,  toolaction(pantool, Number(6)),
+		"pan mode"},
+	{"Wire",   w_xpm,  toolaction(changetool, Number(XCF_Wire)),
+		"draw wire"},
+	{"Box",    b_xpm,  toolaction(changetool, Number(XCF_Box)),
 		"draw box"},
-	{"Arc",    a_xpm,  toolaction(startarc, NULL),
+	{"Arc",    a_xpm,  toolaction(changetool, Number(XCF_Arc)),
 		"draw arc"},
-	{"Spline", s_xpm,  toolaction(startspline, NULL),
+	{"Spline", s_xpm,  toolaction(changetool, Number(XCF_Spline)),
 		"draw spline"},
-	{"Text",   t_xpm,  toolaction(starttext, Number(NORMAL)),
+	{"Text",   t_xpm,  toolaction(changetool, Number(XCF_Text)),
 		"enter text"},
-	{"Copy",   cp_xpm, toolaction(startcopy, NULL),
+	{"Move",   mv_xpm, toolaction(exec_or_changetool, Number(XCF_Move)),
+		"move element"},
+	{"Copy",   cp_xpm, toolaction(exec_or_changetool, Number(XCF_Copy)),
 		"copy element"},
-	{"Edit",   e_xpm,  toolaction(startedit, NULL),
+	{"Edit",   e_xpm,  toolaction(exec_or_changetool, Number(XCF_Edit)),
 		"edit element"},
-	{"Delete", d2_xpm, toolaction(startdelete, NULL),
+	{"Delete", d2_xpm, toolaction(exec_or_changetool, Number(XCF_Delete)),
 		"delete element"},
-	{"RotP",   cw_xpm, toolaction(startrotate, Number(3)),
+	{"RotP",   cw_xpm, toolaction(rotatetool, Number(15)),
 		"rotate 15 degrees clockwise"},
-	{"RotN",  ccw_xpm, toolaction(startrotate, Number(-3)),
+	{"RotN",  ccw_xpm, toolaction(rotatetool, Number(-15)),
 		"rotate 15 degrees counterclockwise"},
-	{"HFlip",  fx_xpm, toolaction(startrotate, Number(512)),
+	{"HFlip",  fx_xpm, toolaction(exec_or_changetool, Number(XCF_Flip_X)),
 		"flip horizontal"},
-	{"VFlip",  fy_xpm, toolaction(startrotate, Number(1024)),
+	{"VFlip",  fy_xpm, toolaction(exec_or_changetool, Number(XCF_Flip_Y)),
 		"flip vertical"},
-	{"Push",  pu2_xpm, toolaction(startpush, NULL),
+	{"Push",  pu2_xpm, toolaction(exec_or_changetool, Number(XCF_Push)),
 		"push (edit object)"},
 	{"Pop",   po2_xpm, toolaction(popobject, Number(0)),
 		"pop (return from object edit)"},
