@@ -41,7 +41,7 @@
 extern Tcl_Interp *xcinterp;
 #endif
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Widget     menuwidgets[];
 extern short textpos, textend;
 extern char _STR[150];
@@ -716,7 +716,7 @@ int psubstitute(objinstptr thisinst)
    objinstptr pinst;
    objectptr thisobj;
 
-   pinst = (thisinst == areastruct.topinstance) ? areastruct.topinstance : thisinst;
+   pinst = (thisinst == areawin->topinstance) ? areawin->topinstance : thisinst;
    if (pinst == NULL) return -1;		/* there is no instance */
    else if (pinst->thisobject->params == NULL)
       return -1;			    /* object has no parameters */
@@ -947,7 +947,7 @@ void pwriteback(objinstptr thisinst)
    /* pwriteback checks for cases in which a change in one element	*/
    /* precipitates a change in another.  If so, force a redraw.		*/
 
-   if (need_redraw && (thisinst == areastruct.topinstance))
+   if (need_redraw && (thisinst == areawin->topinstance))
       drawarea(NULL, NULL, NULL);
 }
 
@@ -1345,10 +1345,10 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 			"a parameter of a different type\n");
 	 goto param_done;
       }
-      if ((newops = match_instance_param(areastruct.topinstance, keyptr)) == NULL) {
+      if ((newops = match_instance_param(areawin->topinstance, keyptr)) == NULL) {
          newops = make_new_parameter(keyptr);
-         newops->next = areastruct.topinstance->params;
-         areastruct.topinstance->params = newops;
+         newops->next = areawin->topinstance->params;
+         areawin->topinstance->params = newops;
          newops->type = ops->type;
          newops->which = ops->which;
       }
@@ -1439,14 +1439,14 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
          case POLYGON:
 	    switch(mode) {
 	       case P_POSITION_X:
-	          pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
+	          pptr = TOPOLY(gelem)->points + areawin->editcycle; 
 	          newops->parameter.ivalue = (int)pptr->x;
-	          epp->pdata.pointno = areastruct.editcycle;
+	          epp->pdata.pointno = areawin->editcycle;
 	          break;
 	       case P_POSITION_Y:
-	          pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
+	          pptr = TOPOLY(gelem)->points + areawin->editcycle; 
 	          newops->parameter.ivalue = (int)pptr->y;
-	          epp->pdata.pointno = areastruct.editcycle;
+	          epp->pdata.pointno = areawin->editcycle;
 	          break;
 	       case P_STYLE:
 	          newops->parameter.ivalue = (int)TOPOLY(gelem)->style;
@@ -1460,14 +1460,14 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
          case SPLINE:
 	    switch(mode) {
 	       case P_POSITION_X:
-	          pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
+	          pptr = TOSPLINE(gelem)->ctrl + areawin->editcycle; 
 	          newops->parameter.ivalue = (int)pptr->x;
-	          epp->pdata.pointno = areastruct.editcycle;
+	          epp->pdata.pointno = areawin->editcycle;
 	          break;
 	       case P_POSITION_Y:
-	          pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
+	          pptr = TOSPLINE(gelem)->ctrl + areawin->editcycle; 
 	          newops->parameter.ivalue = (int)pptr->y;
-	          epp->pdata.pointno = areastruct.editcycle;
+	          epp->pdata.pointno = areawin->editcycle;
 	          break;
 	       case P_STYLE:
 	          newops->parameter.ivalue = (int)TOSPLINE(gelem)->style;
@@ -1663,16 +1663,16 @@ void makeparam(labelptr thislabel, char *key)
    /* intended parameter boundaries				*/
 
    if (textend > 0 && textend < textpos) {  	/* partial string */
-      splitstring(textend, &thislabel->string, areastruct.topinstance);
-      splitstring(textpos, &thislabel->string, areastruct.topinstance);
+      splitstring(textend, &thislabel->string, areawin->topinstance);
+      splitstring(textpos, &thislabel->string, areawin->topinstance);
 
       /* Because "splitstring" changes all the pointers, find the    */
       /* stringpart structures at textend and textpos positions.     */
 
       begpart = findstringpart(textend, NULL, thislabel->string,
-		areastruct.topinstance);
+		areawin->topinstance);
       endpart = findstringpart(textpos, NULL, thislabel->string,
-		areastruct.topinstance);
+		areawin->topinstance);
 
       /* Make the new segments for PARAM_START and PARAM_END.	*/
 
@@ -1922,11 +1922,11 @@ void unparameterize(int mode)
    else
       ptype = ALL_TYPES;
 
-   if ((areastruct.selects == 1) && (mode == P_SUBSTRING) && textend > 0
+   if ((areawin->selects == 1) && (mode == P_SUBSTRING) && textend > 0
 		&& textend < textpos) {
-      if (SELECTTYPE(areastruct.selectlist) != LABEL) return;	 /* Not a label */
-      settext = SELTOLABEL(areastruct.selectlist);
-      strptr = findstringpart(textend, &locpos, settext->string, areastruct.topinstance);
+      if (SELECTTYPE(areawin->selectlist) != LABEL) return;	 /* Not a label */
+      settext = SELTOLABEL(areawin->selectlist);
+      strptr = findstringpart(textend, &locpos, settext->string, areawin->topinstance);
       while (strptr != NULL && strptr->type != PARAM_END)
 	 strptr = strptr->nextpart;
       if (strptr == NULL) return;	/* No parameters */
@@ -1937,16 +1937,16 @@ void unparameterize(int mode)
       /* the whole parameter or the parameter start marker.		  */
 
       for (tmpptr = settext->string; tmpptr != NULL && tmpptr != strptr;
-		tmpptr = nextstringpart(tmpptr, areastruct.topinstance))
+		tmpptr = nextstringpart(tmpptr, areawin->topinstance))
 	 if (tmpptr->type == PARAM_START) lastptr = tmpptr;
       /* Finish search, unlinking any parameter we might be inside */
-      for (; tmpptr != NULL; tmpptr = nextstringpart(tmpptr, areastruct.topinstance));
+      for (; tmpptr != NULL; tmpptr = nextstringpart(tmpptr, areawin->topinstance));
 
       if (lastptr != NULL) unmakeparam(settext, lastptr);
    }
    else {
-      for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
-            areastruct.selects; fselect++) {
+      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
+            areawin->selects; fselect++) {
          if ((mode == P_SUBSTRING) && SELECTTYPE(fselect) == LABEL) {
             settext = SELTOLABEL(fselect);
             strptr = settext->string;
@@ -1982,9 +1982,9 @@ void parameterize(int mode, char *key)
    else
       ptype = ALL_TYPES;
 
-   for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
-            areastruct.selects; fselect++) {
-      if ((mode == P_SUBSTRING) && (areastruct.selects == 1) &&
+   for (fselect = areawin->selectlist; fselect < areawin->selectlist +
+            areawin->selects; fselect++) {
+      if ((mode == P_SUBSTRING) && (areawin->selects == 1) &&
 		(SELECTTYPE(fselect) == LABEL)) {
          settext = SELTOLABEL(fselect);
          makeparam(settext, key);
@@ -2010,7 +2010,7 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
    stringpart *firstptr, *lastptr;
    int locpos;
 
-   lastptr = findstringpart(textpos, &locpos, tlab->string, areastruct.topinstance);
+   lastptr = findstringpart(textpos, &locpos, tlab->string, areawin->topinstance);
 
    /* This text position can't be inside another parameter */
    for (firstptr = lastptr; firstptr != NULL; firstptr = firstptr->nextpart)
@@ -2019,7 +2019,7 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
    /* The area between textend and textpos cannot contain a parameter */
    if (textend > 0)
       for (firstptr = findstringpart(textend, &locpos, tlab->string,
-		areastruct.topinstance); firstptr != lastptr;
+		areawin->topinstance); firstptr != lastptr;
 		firstptr = firstptr->nextpart)
          if (firstptr->type == PARAM_START || firstptr->type == PARAM_END)
 	    return True;
@@ -2036,7 +2036,7 @@ int checklibtop()
    int i;
    pushlistptr thispush;
 
-   for (thispush = areastruct.stack; thispush != NULL; thispush = thispush->next)
+   for (thispush = areawin->stack; thispush != NULL; thispush = thispush->next)
       if ((i = is_library(thispush->thisinst->thisobject)) >= 0)
 	 return i;
 
