@@ -70,8 +70,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 extern Globaldata xobjs;
-extern Clientdata areastruct;
-extern short eventmode;
+extern XCWindowData *areawin;
 
 /*----------------------------------------------------------------------*/
 /* remember_selection ---						*/
@@ -105,7 +104,7 @@ uselection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areastruct from the saved uselection	*/
+/* Create a selection list in areawin->from the saved uselection	*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
@@ -250,8 +249,8 @@ int select_previous(Undoptr thisrecord)
 	 case XCF_Copy:
 	 case XCF_Select:
 	    srec = (uselection *)chkrecord->undodata;
-	    areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
-	    areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
+	    areawin->selectlist = regen_selection(thisrecord->thisinst, srec);
+	    areawin->selects = (areawin->selectlist) ? srec->number : 0;
 	    return 0;
       }
    }
@@ -428,7 +427,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 /* 1 arg:							*/
 	 /*	egen = element just created (genericptr)		*/
 	 egen = va_arg(args, genericptr);
@@ -447,7 +446,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	    case LABEL:
 	       erec->save.string =
 			stringcopyall(((labelptr)egen)->string,
-			areastruct.topinstance);
+			areawin->topinstance);
 	       newrecord->idata = ((labelptr)egen)->justify;
 	       break;
 	    case POLYGON:
@@ -547,12 +546,12 @@ short undo_one_action()
       case XCF_Delete:
 	 unselect_all();
 	 thisobj = (objectptr)thisrecord->undodata;
-	 areastruct.selects = thisobj->parts;
-	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
+	 areawin->selects = thisobj->parts;
+	 areawin->selectlist = xc_undelete(thisrecord->thisinst,
 			thisobj, (short)thisrecord->idata,
 			recover_selectlist(thisrecord));
-	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
-			areastruct.selects);
+	 srec = remember_selection(thisrecord->thisinst, areawin->selectlist,
+			areawin->selects);
 	 thisrecord->undodata = (char *)srec;
 	 draw_all_selected();
 	 break;
@@ -577,18 +576,18 @@ short undo_one_action()
 	    delete_one_element(thisrecord->thisinst, egen);
 
 	 /* Put back all the parts */
-	 areastruct.selects = thisobj->parts;
-	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
+	 areawin->selects = thisobj->parts;
+	 areawin->selectlist = xc_undelete(thisrecord->thisinst,
 			thisobj, (short)thisrecord->idata,
 			recover_selectlist(thisrecord));
-	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
-			areastruct.selects);
+	 srec = remember_selection(thisrecord->thisinst, areawin->selectlist,
+			areawin->selects);
 	 thisrecord->undodata = (char *)srec;
 	 draw_all_selected();
 	 break;
 
       case XCF_Push:
-	 popobject(areastruct.area, 0, NULL);
+	 popobject(areawin->area, 0, NULL);
 	 break;
 
       case XCF_Pop:
@@ -610,7 +609,7 @@ short undo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
 		case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-		case XCF_Spline: case XCF_Dot:
+		case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 egen = (genericptr)thisrecord->undodata;
 	 i = thisrecord->thisinst->thisobject->parts - 1;
 	 if (egen != *(thisrecord->thisinst->thisobject->plist + i)) {
@@ -619,7 +618,7 @@ short undo_one_action()
 	 }
 	 else {
 	    delete_one_element(thisrecord->thisinst, egen);
-	    drawarea(areastruct.area, NULL, NULL);
+	    drawarea(areawin->area, NULL, NULL);
 	 }
 	 break;
 
@@ -651,7 +650,7 @@ short undo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
 	       pointlist tmppts;
@@ -662,7 +661,7 @@ short undo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case ARC: {
 	       arcinfo tmpinfo;
@@ -677,7 +676,7 @@ short undo_one_action()
 	       earc->yaxis = erec->save.arcspecs->yaxis;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
 	 break;
@@ -701,28 +700,28 @@ short undo_one_action()
 
 	 /* Revert selection to previously selected */
 	 select_previous(thisrecord);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Flip_X:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementflip();
 	 break;
 
       case XCF_Flip_Y:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementvflip();
 	 break;
 
       case XCF_Rotate:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementrotate(-thisrecord->idata);
 	 break;
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
 	 placeselects(-(delta->x), -(delta->y), NULL);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       default:
@@ -792,7 +791,7 @@ short redo_one_action()
 	 thisrecord->undodata = (char *)thisobj;
 	 thisrecord->idata = (int)DRAW;
 	 unselect_all();
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       /* Unfinished! */
@@ -806,7 +805,7 @@ short redo_one_action()
 	 break;
 
       case XCF_Pop:
-	 popobject(areastruct.area, 0, NULL);
+	 popobject(areawin->area, 0, NULL);
 	 break;
 
       case XCF_Push:
@@ -816,8 +815,8 @@ short redo_one_action()
       case XCF_Select:
 	 unselect_all();
 	 srec = (uselection *)thisrecord->undodata;
-	 areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
-	 areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
+	 areawin->selectlist = regen_selection(thisrecord->thisinst, srec);
+	 areawin->selects = (areawin->selectlist) ? srec->number : 0;
 	 draw_all_selected();
 	 break;
 
@@ -837,21 +836,21 @@ short redo_one_action()
 	 thisobj = (objectptr)thisrecord->undodata;
 	 if (thisobj != NULL) {
 	    unselect_all();
-	    areastruct.selects = thisobj->parts;
-	    areastruct.selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW,
+	    areawin->selects = thisobj->parts;
+	    areawin->selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW,
 			NULL);
 	    thisrecord->undodata = (char *)remember_selection(thisrecord->thisinst,
-			areastruct.selectlist, areastruct.selects);
+			areawin->selectlist, areawin->selects);
 	    draw_all_selected();
 	 }
 	 break;
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Edit:
@@ -882,7 +881,7 @@ short redo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
 	       pointlist tmppts;
@@ -893,7 +892,7 @@ short redo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case ARC: {
 	       arcinfo tmpinfo;
@@ -908,30 +907,30 @@ short redo_one_action()
 	       earc->yaxis = erec->save.arcspecs->yaxis;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
 	 break;
 
       case XCF_Flip_X:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementflip();
 	 break;
 
       case XCF_Flip_Y:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementvflip();
 	 break;
 
       case XCF_Rotate:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
+	 areawin->save = *((XPoint *)thisrecord->undodata);
 	 elementrotate(thisrecord->idata);
 	 break;
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
 	 placeselects(delta->x, delta->y, NULL);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       default:
@@ -1036,7 +1035,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 /* if MODE_UNDO, the element is on the page, so don't destroy it! */
 	 if (mode == MODE_REDO)
 	    free(thisrecord->undodata);
